/**
 * User Class represent a a user that contains user color , user name ,available blocks that user can get and ...
 * this class has some methods to know user can take a block or not
 *
 * @author zahra_fatehi
 * @version 0
 */

import java.util.ArrayList;

public class User {
    private String name;
    private ArrayList<Block> userBlocks;
    private ArrayList<Block> availableBlocksToPick;
    private Character color;

    /**
     * create a list of user's block and a list of blocks that user can take
     */
    public User(){
        userBlocks = new ArrayList<Block>();
        availableBlocksToPick = new ArrayList<Block>();
    }

    /**
     * add a block to taken block list for this user
     * @param block is a block we wanna add to taken blocks
     */
    public void addBlock(Block block){
        userBlocks.add(block);
    }

    /**
     *remove a block from taken block list for this user
     *@param block is a block we wanna remove from taken blocks
     */
    public void removeBlock(Block block){
        userBlocks.remove(block);
    }

    /**
     * check a block is available to take for this user or not
     * @param block is a block that user wanna take
     * @param blocks is the list of all 64 blocks
     * @return true if is available to take the block and false if it is not
     */
    public boolean isAvailableBlockToPick(Block block , ArrayList<Block> blocks ) {
        int i, j;
        int flag1 = 0;// for checking neighbors has same color in each step or not
        int flag2 = 0;// to not change " w b b b w b b w " to "w w w w w w w w"
        int flag3 = 0;// to know you can pick an empty block with help of checking changing color
        for (Block b : blocks){
            if ( b.getColor().equals('n') == false && block.getRow() == b.getRow() && block.getColumn() == b.getColumn()){
                return false;
            }
        }

        //check down
        flag1 = 0;
        flag2 = 0;
        for (i = block.getRow() + 1 ; i < 8; i++) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getColumn() == block.getColumn() && block1.getRow() == block.getRow() + 1 && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1 || flag2 == 1) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getRow() == i && block1 .getColumn() == block.getColumn() ){//to not change "b w n w b" to "b b b b b"
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1 ) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getRow() == i && block1.getColumn() == block.getColumn() && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }


        //check up
        flag1 = 0;
        flag2 = 0;
        for (i = block.getRow() - 1; i > -1; i--) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getRow() == block.getRow() - 1 && block1.getColumn() == block.getColumn() && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getRow() == i && block1.getColumn() == block.getColumn()){//to not change "b w n w b" to "b b b b b"
                    flag1= 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getRow() == i && block1.getColumn() == block.getColumn() && block1.getColor().equals(color)) {

                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }

        //check right
        flag1 = 0;//check neighbor
        flag2 = 0;
        for (i = block.getColumn() + 1; i < 8; i++) {// find the next block with chosen block color
            for (Block block1 : blocks) {//check neighbor
                if (block1.getColumn() == block.getColumn() + 1 && block1.getRow() == block.getRow() && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1)
                break;
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == block.getRow()){//to not change "b w n w b" to "b b b b b"
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == block.getRow() && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }


        //check left
        flag1 = 0;
        flag2 = 0;
        for (i = block.getColumn() - 1; i > -1; i--) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getColumn() == block.getColumn() - 1 && block1.getRow() == block.getRow() && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1)
                break;
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i &&  block1.getRow() == block.getRow()){//to not change "b w n w b" to "b b b b b"
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == block.getRow() && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;

                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }


        //check up and right
        flag1 = 0;
        flag2 = 0;
        for (i = block.getColumn() + 1, j = block.getRow() - 1; i < 8 && j > -1; i++, j--) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getColumn() == block.getColumn() + 1 && block1.getRow() == block.getRow() - 1 && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j){//to not change "b w n w b" to "b b b b b"
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }


        //check down and left
        flag1 = 0;
        flag2 = 0;
        for (i = block.getColumn() - 1, j = block.getRow() + 1; i > -1 && j < 8; i--, j++) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getColumn() == block.getColumn() - 1 && block1.getRow() == block.getRow() + 1 && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if (flag1 == 1) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j ){//to not change "b w n w b" to "b b b b b"
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }


        //check down and right
        flag1 = 0;
        flag2 = 0;
        for (i = block.getColumn() + 1, j = block.getRow() + 1; i < 8 && j < 8; i++, j++) {// find the next block with chosen block color
            for (Block block1 : blocks) {

                if (block1.getColumn() == block.getColumn() + 1 && block1.getRow() == block.getRow() + 1 && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if ( flag1 == 1) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j ){//to not change "b w n w b" to "b b b b b"
                    flag1  = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 2)
                break;
        }


        //check up and left
        flag1 = 0;
        flag2 = 0;
        for (i = block.getColumn() - 1, j = block.getRow() - 1; i > -1 && j > -1; i--, j--) {// find the next block with chosen block color
            for (Block block1 : blocks) {
                if (block1.getColumn() == block.getColumn() - 1 && block1.getRow() == block.getRow() - 1 && (block1.getColor().equals(color) || block1.getColor().equals('n'))) {
                    flag1 = 1;
                    break;
                }
            }
            if ( flag1 == 1 ) {
                break;
            }
            for (Block block1 : blocks){
                if (block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j){
                    flag1 = 1;
                    break;
                }
                if (flag2 == 1) {// to not change " w b b b w b b w " to "w w w w w w w w"
                    break;
                }
                if (!block1.getColor().equals('n') && block1.getColumn() == i && block1.getRow() == j && block1.getColor().equals(color)) {
                    flag2 = 1;
                    flag3 = 1;
                }
            }
            if (flag1 == 1 || flag2 == 1)
                break;
        }

        if (flag3 == 1) {
            return true;
        }
        return false;
    }

    /**
     * add block to list of available blocks user to take
     * @param block is a block that is available for user to take
     */
    public void addAvailableBlockToPick(Block block){
        availableBlocksToPick.add(block);
    }

    /**
     *remove block from list of available blocks user to take
     *@param block is a block that is available for user to take
     */
    public void removeAvailableBlockToPick(Block block){
        availableBlocksToPick.remove(block);
    }

    /**
     * @return color of user
     */
    public Character getColor() {
        return color;
    }

    /**
     * @param color is color of user
     */
    public void setColor(Character color) {
        this.color = color;
    }

    /**
     * @return name of user
     */
    public String getName() {
        return name;
    }

    /**
     * @param name is the name of user
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the list of available blocks to take
     */
    public ArrayList<Block> getAvailableBlocksToPick() {
        return availableBlocksToPick;
    }

    /**
     * @return the list of blocks that user have
     */
    public ArrayList<Block> getUserBlocks() {
        return userBlocks;
    }

}

